@@ -106,8 +106,13 @@ error_messages = {
 
 
 class GroongaError(Exception):
-    def __init__(self, err):
-        self.msg = error_messages.get(err) or err
+    def __init__(self, err, reason="", cause=""):
+        self.errmsg = error_messages.get(err) or err
+        self.reason = reason
+        self.cause  = cause
 
     def __str__(self):
-        return self.msg
+        msg = "%s: %s" % (self.errmsg, self.reason)
+        if self.cause:
+            msg += ", query is `%s`" % self.cause
+        return msg
