@@ -35,6 +35,7 @@ import json
 import logging
 
 from pyroonga import utils
+from pyroonga.orm.attributes import SuggestType
 
 logger = logging.getLogger(__name__)
 
@@ -59,116 +60,17 @@ class Query(object):
         self._table = tbl
 
 
-class GroongaResultBase(object):
-    """Base class of query result"""
+class QueryOptionsMixin(object):
+    __options__ = {'limit':  '--limit',
+                   'offset': '--offset',
+                   'sortby': '--sortby',
+                   'output_columns': '--output_columns'}
 
-    def __init__(self, cls, results, maxlen=None):
-        """Construct of GroongaResultBase
-
-        :param cls: Class for mappings.
-        :param results: query results.
-        :param maxlen: maximum length of mapping results. Default is all.
-        """
-        cols = [col[0] for col in results[1]]
-        colrange = range(len(cols))
-        result = []
-        for v in results[2:maxlen]:
-            mapped = dict(zip(cols, [v[i] for i in colrange]))
-            result.append(cls(**mapped))
-        self._all_len = results[0][0]
-        self._result = result
-
-    @property
-    def all_len(self):
-        """All length of query results
-
-        Note that the number of elements is not necessarily equal.
-        """
-        return self._all_len
-
-    def __len__(self):
-        return len(self._result)
-
-    def __iter__(self):
-        return iter(self._result)
-
-    def __getitem__(self, key):
-        return self._result[key]
-
-    def __reversed__(self):
-        return reversed(self._result)
-
-
-class GroongaResult(GroongaResultBase):
-    """Result class for 'select' query"""
-
-    def __init__(self, table, resultstr, maxlen=None):
-        """Construct of GroongaResult
-
-        :param table: Table class for mappings.
-        :param resultstr: result string of 'select' query.
-        :param maxlen: maximum length of mapping results. Default is all.
-        """
-        objs = json.loads(resultstr)
-        super(GroongaResult, self).__init__(table, objs[0], maxlen)
-        self._drilldown = self._drilldown_mapping(objs[1:])
-        self._table = table
-
-    def _drilldown_mapping(self, results):
-        drilldown = [GroongaDrilldownResult(Drilldown, v) for v in results]
-        return drilldown
-
-    @property
-    def drilldown(self):
-        """List of instance of `GroongaDrilldownResult`"""
-        return self._drilldown
-
-
-class GroongaDrilldownResult(GroongaResultBase):
-    """Result class for drilldown"""
-
-
-class Drilldown(object):
-    """Drilldown representation class"""
-
-    def __init__(self, _key=None, _nsubrecs=None):
-        self._key = _key
-        self._nsubrecs = _nsubrecs
-
-
-class SelectQueryBase(Query):
-    """'select' query representation base class"""
-
-    __options__ = {'limit':  '',
-                   'offset': '',
-                   'sortby': '',
-                   'output_columns': ''}
-
-    def __init__(self, tbl, *args, **kwargs):
-        """Construct of 'select' query
-
-        :param tbl: Table class. see also :class:`Query`\ .
-        :param args: :class:`ExpressionTree`\ .
-        :param kwargs: search columns and search texts.
-        """
-        super(SelectQueryBase, self).__init__(tbl)
-        self._expr = args
-        self._target = kwargs
+    def __init__(self):
         self._limit = None
         self._offset = None
         self._sortby = []
         self._output_columns = []
-        self._cache = True
-        self._match_escalation_threshold = None
-
-    def all(self):
-        """Obtain the all result from this query instance
-
-        :returns: result of query as a Python's objects. (dict, list, etc...)
-        """
-        q = str(self)
-        result = self._table.grn.query(q)
-        return GroongaResult(self._table, result)
 
     def limit(self, lim):
         """Limit for number of result of query
@@ -206,24 +108,6 @@ class SelectQueryBase(Query):
         self._output_columns = args
         return self
 
-    def cache(self, iscache):
-        """Set the query cache
-
-        :param iscache: no query cache if False. otherwise **yes**\ .
-        :returns: self. for method chain.
-        """
-        self._cache = bool(iscache)
-        return self
-
-    def match_escalation_threshold(self, threshold):
-        """Set the match escalation threshold
-
-        :param threshold: threshold of match escalation
-        :returns: self. for method chain.
-        """
-        self._match_escalation_threshold = int(threshold)
-        return self
-
     def _makelimit(self):
         if self._limit:
             return '%s %d' % (self.__options__['limit'], self._limit)
@@ -251,6 +135,163 @@ class SelectQueryBase(Query):
         else:
             return ''
 
+    def _condition(self):
+        return '%(limit)s %(offset)s %(sortby)s %(output_columns)s' % \
+               dict(limit=self._makelimit(),
+                    offset=self._makeoffset(),
+                    sortby=self._makesortby(),
+                    output_columns=self._makeoutput_columns())
+
+
+class GroongaResultBase(object):
+    """Base class of query result"""
+
+    def __init__(self, cls, results, maxlen=None):
+        """Construct of GroongaResultBase
+
+        :param cls: Class for mappings.
+        :param results: query results.
+        :param maxlen: maximum length of mapping results. Default is all.
+        """
+        cols = [col[0] for col in results[1]]
+        colrange = range(len(cols))
+        result = []
+        # TODO: implements by generator
+        for v in results[2:maxlen]:
+            mapped = dict(zip(cols, [v[i] for i in colrange]))
+            result.append(cls(**mapped))
+        self._all_len = results[0][0]
+        self._result = result
+
+    @property
+    def all_len(self):
+        """All length of query results
+
+        Note that the number of elements is not necessarily equal.
+        """
+        return self._all_len
+
+    def __len__(self):
+        return len(self._result)
+
+    def __iter__(self):
+        return iter(self._result)
+
+    def __getitem__(self, key):
+        return self._result[key]
+
+    def __reversed__(self):
+        return reversed(self._result)
+
+
+class GroongaSelectResult(GroongaResultBase):
+    """Result class for 'select' query"""
+
+    def __init__(self, table, resultstr, maxlen=None):
+        """Construct of GroongaSelectResult
+
+        :param table: Table class for mappings.
+        :param resultstr: result string of 'select' query.
+        :param maxlen: maximum length of mapping results. Default is all.
+        """
+        objs = json.loads(resultstr)
+        super(GroongaSelectResult, self).__init__(table, objs[0], maxlen)
+        self._drilldown = self._drilldown_mapping(objs[1:])
+        self._table = table
+
+    def _drilldown_mapping(self, results):
+        drilldown = [GroongaDrilldownResult(Drilldown, v) for v in results]
+        return drilldown
+
+    @property
+    def drilldown(self):
+        """List of instance of `GroongaDrilldownResult`"""
+        return self._drilldown
+
+
+class GroongaDrilldownResult(GroongaResultBase):
+    """Result class for drilldown"""
+
+
+class GroongaSuggestResults(object):
+    """Results of suggestion representation class"""
+
+    __slots__ = ['complete', 'correct', 'suggest']
+
+    def __init__(self, resultstr):
+        result = json.loads(resultstr)
+        complete = result.get('complete', [])
+        correct  = result.get('correct', [])
+        suggest  = result.get('suggest', [])
+        self.complete = complete and GroongaSuggestResult(Suggest, complete)
+        self.correct  = correct  and GroongaSuggestResult(Suggest, correct)
+        self.suggest  = suggest  and GroongaSuggestResult(Suggest, suggest)
+
+
+class GroongaSuggestResult(GroongaResultBase):
+    """Result class for suggest"""
+
+
+class Suggest(object):
+    """Suggest representation class"""
+
+    def __init__(self, _key=None, _score=None):
+        self._key = _key
+        self._score = _score
+
+
+class Drilldown(object):
+    """Drilldown representation class"""
+
+    def __init__(self, _key=None, _nsubrecs=None):
+        self._key = _key
+        self._nsubrecs = _nsubrecs
+
+
+class SelectQueryBase(Query, QueryOptionsMixin):
+    """'select' query representation base class"""
+
+    def __init__(self, tbl, *args, **kwargs):
+        """Construct of 'select' query
+
+        :param tbl: Table class. see also :class:`Query`\ .
+        :param args: :class:`ExpressionTree`\ .
+        :param kwargs: search columns and search texts.
+        """
+        Query.__init__(self, tbl)
+        QueryOptionsMixin.__init__(self)
+        self._expr = args
+        self._target = kwargs
+        self._cache = True
+        self._match_escalation_threshold = None
+
+    def all(self):
+        """Obtain the all result from this query instance
+
+        :returns: result of query as a Python's objects. (dict, list, etc...)
+        """
+        q = str(self)
+        result = self._table.grn.query(q)
+        return GroongaSelectResult(self._table, result)
+
+    def cache(self, iscache):
+        """Set the query cache
+
+        :param iscache: no query cache if False. otherwise **yes**\ .
+        :returns: self. for method chain.
+        """
+        self._cache = bool(iscache)
+        return self
+
+    def match_escalation_threshold(self, threshold):
+        """Set the match escalation threshold
+
+        :param threshold: threshold of match escalation
+        :returns: self. for method chain.
+        """
+        self._match_escalation_threshold = int(threshold)
+        return self
+
     def _makecache(self):
         return '' if self._cache else '--cache no'
 
@@ -265,13 +306,10 @@ class SelectQueryBase(Query):
         return ''
 
     def _condition(self):
-        return '%(cache)s %(limit)s %(offset)s %(sortby)s %(output_columns)s ' \
-               '%(match_escalation_threshold)s %(params)s' % \
-               dict(cache=self._makecache(),
-                    limit=self._makelimit(),
-                    offset=self._makeoffset(),
-                    sortby=self._makesortby(),
-                    output_columns=self._makeoutput_columns(),
+        return '%(condition)s %(cache)s %(match_escalation_threshold)s ' \
+               '%(params)s' % \
+               dict(condition=QueryOptionsMixin._condition(self),
+                    cache=self._makecache(),
                     match_escalation_threshold=self._makematch_escalation_threshold(),
                     params=self._makeparams())
 
@@ -284,11 +322,6 @@ class SelectQueryBase(Query):
 class SelectQuery(SelectQueryBase):
     """Query representation class for 'select' query"""
 
-    __options__ = {'limit':  '--limit',
-                   'offset': '--offset',
-                   'sortby': '--sortby',
-                   'output_columns': '--output_columns'}
-
     def drilldown(self, *columns):
         """Switch to the drilldown query
 
@@ -319,7 +352,7 @@ class SelectQuery(SelectQueryBase):
             return str(expr)
 
 
-class DrillDownQuery(SelectQueryBase):
+class DrillDownQuery(SelectQueryBase, QueryOptionsMixin):
     """'select' query with drilldown representation class
 
     Instantiate from :meth:`SelectQuery.drilldown`\ .
@@ -339,7 +372,8 @@ class DrillDownQuery(SelectQueryBase):
         """
         if not args:
             raise ValueError("args is must be one or more columns")
-        super(DrillDownQuery, self).__init__(parent._table)
+        SelectQueryBase.__init__(self, parent._table)
+        QueryOptionsMixin.__init__(self)
         self.parent = parent
         self.columns = args
 
@@ -460,3 +494,149 @@ class LoadQuery(Query):
         return 'load --table %(table)s --input_type json --values ' \
                '"%(data)s"' % dict(table=self._table.__name__,
                                    data=self._makejson())
+
+
+class SuggestQuery(Query, QueryOptionsMixin):
+    """'suggest' query representation class"""
+
+    __options__ = {'limit':  '--limit',
+                   'offset': '--offset',
+                   'sortby': '--sortby',
+                   'output_columns': '--output_columns',
+                   'frequency_threshold': '--frequency_threshold',
+                   'conditional_probability_threshold':
+                       '--conditional_probability_threshold',
+                   'prefix_search': '--prefix_search'}
+
+    def __init__(self, tbl, query):
+        """Construct of 'suggest' query
+
+        :param tbl: :class:`pyroonga.orm.table.item_query` class.
+            see also :class:`Query`\ .
+        :param query: query string for suggest.
+        """
+        Query.__init__(self, tbl)
+        QueryOptionsMixin.__init__(self)
+        self._query = query
+        self._types = SuggestType.complete
+        self._frequency_threshold = None
+        self._conditional_probability_threshold = None
+        self._prefix_search = None
+        self._result = None
+
+    def all(self):
+        """Get results of suggest
+
+        :returns: :class:`GroongaSuggestResults`
+        """
+        query = str(self)
+        result = self._table.grn.query(query)
+        return GroongaSuggestResults(result)
+
+    def get(self, type_):
+        """Get a result of suggest by type name
+
+        :param type_: type name of suggest. 'complete', 'correct' or 'suggest'.
+        :returns: :class:`GroongaSuggestResult`\ .
+        :raises: KeyError
+        """
+        result = self.all()
+        try:
+            return getattr(result, type_)
+        except AttributeError:
+            raise KeyError(type_)
+
+    def __getitem__(self, key):
+        """Get results of suggest by type name
+
+        Same as :meth:`SuggestQuery.get`
+        """
+        return self.get(key)
+
+    def types(self, types):
+        """Set the suggestion types
+
+        :param types: see :class:`pyroonga.orm.attributes.SuggestType`
+        :returns: self. for method chain.
+        """
+        self._types = types
+        return self
+
+    def frequency_threshold(self, threshold):
+        """Set the frequency threshold
+
+        :param threshold: threshold of frequency
+        :returns: self. for method chain.
+        """
+        self._frequency_threshold = int(threshold)
+        return self
+
+    def conditional_probability_threshold(self, threshold):
+        """Set the conditional probability threshold
+
+        :param threshold: threshold of conditional probability
+        :returns: self. for method chain.
+        """
+        self._conditional_probability_threshold = float(threshold)
+        return self
+
+    def prefix_search(self, isprefixsearch):
+        """Set the prefix search
+
+        :param threshold: threshold of conditional probability
+        :param isprefixsearch: It specifies whether optional prefix search is
+            used or not in completion. 'yes' if True, otherwise 'no'.
+        :returns: self. for method chain.
+        """
+        self._prefix_search = bool(isprefixsearch)
+        return self
+
+    def _makefrequency_threshold(self):
+        if self._frequency_threshold is None:
+            return ''
+        else:
+            return ('%s %d' % (self.__options__['frequency_threshold'],
+                               self._frequency_threshold))
+
+    def _makeconditional_probability_threshold(self):
+        if self._conditional_probability_threshold is None:
+            return ''
+        else:
+            return ('%s %.1f' %
+                    (self.__options__['conditional_probability_threshold'],
+                     self._conditional_probability_threshold))
+
+    def _makeprefix_search(self):
+        if self._prefix_search is None:
+            return ''
+        else:
+            return ('%s %s' % (self.__options__['prefix_search'],
+                               'yes' if self._prefix_search else 'no'))
+
+    def _condition(self):
+        return ('%(condition)s %(frequency_threshold)s ' \
+                '%(conditional_probability_threshold)s %(prefix_search)s' % \
+                dict(condition=QueryOptionsMixin._condition(self),
+                     frequency_threshold=self._makefrequency_threshold(),
+                     conditional_probability_threshold=
+                         self._makeconditional_probability_threshold(),
+                     prefix_search=self._makeprefix_search())).strip()
+
+    def __str__(self):
+        return 'suggest --table "%(table)s" --column "%(column)s" --types ' \
+                '"%(types)s" %(condition)s --query "%(query)s"' % \
+                dict(table=self._table.__tablename__,
+                     column=self._table.kana.name,
+                     types=self._types,
+                     condition=self._condition(),
+                     query=utils.escape(self._query))
+
+
+class SuggestLoadQuery(LoadQuery):
+    """'load' query for suggestion representation class"""
+
+    def __str__(self):
+        return 'load --table %(table)s --input_type json --each ' \
+               '\'suggest_preparer(_id, type, item, sequence, time, ' \
+               'pair_query)\' "%(data)s"' % dict(table=self._table.__name__,
+                                                 data=self._makejson())
