@@ -28,15 +28,34 @@
 __author__ = "Naoya INADA <naoina@kuune.org>"
 
 import json
+import itertools
 
 from subprocess import Popen, PIPE
 
 from pyroonga import Groonga
-from pyroonga.orm.attributes import (COLUMN_SCALAR, COLUMN_VECTOR,
-                                     TABLE_HASH_KEY, TABLE_PAT_KEY, ShortText,
-                                     UInt32)
-from pyroonga.orm.table import Column, prop_attr, tablebase
-from pyroonga.orm.query import GroongaResultBase, LoadQuery
+from pyroonga.orm.attributes import (
+    TableFlags,
+    ColumnFlags,
+    DataType,
+    Tokenizer,
+    SuggestType,
+    )
+from pyroonga.orm.table import (
+    Column,
+    TableMeta,
+    SuggestTable,
+    prop_attr,
+    tablebase,
+    event_query,
+    item_query,
+    )
+from pyroonga.orm.query import (
+    GroongaResultBase,
+    GroongaSuggestResults,
+    GroongaSuggestResult,
+    LoadQuery,
+    SuggestQuery,
+    )
 from pyroonga.tests import unittest
 from pyroonga.tests import GroongaTestBase
 
@@ -44,46 +63,72 @@ from pyroonga.tests import GroongaTestBase
 class TestColumn(unittest.TestCase):
     def test___init__with_default_value(self):
         col = Column()
-        self.assertIs(col.flags, COLUMN_SCALAR)
-        self.assertIs(col.type, ShortText)
+        self.assertIs(col.flags, ColumnFlags.COLUMN_SCALAR)
+        self.assertIs(col.type, DataType.ShortText)
+        self.assertIs(col.__tablemeta__, TableMeta)
         self.assertIsNone(col.tablename)
         self.assertIsNone(col.name)
         self.assertIsNone(col.value)
 
     def test___init__(self):
         self.assertRaises(TypeError, Column, flags='')
-        self.assertRaises(TypeError, Column, type='')
         try:
-            col = Column(flags=COLUMN_VECTOR)
-            self.assertIs(col.flags, COLUMN_VECTOR)
+            col = Column(flags=ColumnFlags.COLUMN_VECTOR)
+            self.assertIs(col.flags, ColumnFlags.COLUMN_VECTOR)
         except TypeError:
             self.fail("TypeError has been raised")
 
         try:
-            col = Column(type=UInt32)
-            self.assertIs(col.type, UInt32)
+            col = Column(type=DataType.UInt32)
+            self.assertIs(col.type, DataType.UInt32)
+        except TypeError:
+            self.fail("TypeError has been raised")
+
+        try:
+            col = Column(type=TableMeta)
+            self.assertIs(col.type, TableMeta)
         except TypeError:
             self.fail("TypeError has been raised")
 
     def test___str__(self):
-        col = Column(flags=COLUMN_SCALAR, type=ShortText)
+        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
         self.assertRaises(TypeError, col.__str__)
 
-        col = Column(flags=COLUMN_SCALAR, type=ShortText)
+        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
         col.tablename = 'tb1'
         self.assertRaises(TypeError, col.__str__)
 
-        col = Column(flags=COLUMN_SCALAR, type=ShortText)
+        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
         col.name = 'name1'
         self.assertRaises(TypeError, col.__str__)
 
-        col = Column(flags=COLUMN_VECTOR, type=ShortText)
+        col = Column(flags=ColumnFlags.COLUMN_VECTOR, type=DataType.ShortText)
         col.tablename = 'tb2'
         col.name = 'name2'
         self.assertEqual(col.__str__(), 'column_create --table tb2 ' \
                 '--name name2 --flags COLUMN_VECTOR --type ShortText')
 
 
+        ExampleTableBase = tablebase()
+        class ExampleTable(ExampleTableBase):
+            name = Column()
+
+        col = Column(flags=ColumnFlags.COLUMN_INDEX, type=ExampleTable,
+                     source=ExampleTable.name)
+        col.tablename = 'tb3'
+        col.name = 'name3'
+        self.assertEqual(col.__str__(), 'column_create --table tb3 ' \
+                '--name name3 --flags COLUMN_INDEX --type ExampleTable ' \
+                '--source name')
+
+        col = Column(flags=ColumnFlags.COLUMN_INDEX, type='ExampleTable', source='name')
+        col.tablename = 'tb4'
+        col.name = 'name4'
+        self.assertEqual(col.__str__(), 'column_create --table tb4 ' \
+                '--name name4 --flags COLUMN_INDEX --type ExampleTable ' \
+                '--source name')
+
+
 class TestPropAttr(unittest.TestCase):
     def test_prop_attr(self):
         class TestClass(object):
@@ -172,45 +217,65 @@ class TestTable(GroongaTestBase):
 
     def test_default_value(self):
         Table = tablebase()
-        self.assertIs(Table.__tableflags__, TABLE_HASH_KEY)
-        self.assertIs(Table.__key_type__, ShortText)
+        self.assertIs(Table.__tableflags__, TableFlags.TABLE_HASH_KEY)
+        self.assertIs(Table.__key_type__, DataType.ShortText)
         self.assertEqual(Table.__tablename__, 'Table')
+        self.assertEqual(Table.__default_tokenizer__, None)
 
     def test_table(self):
         Table = tablebase()
-        namecol = Column(flags=COLUMN_SCALAR, type=ShortText)
-        passwordcol = Column(flags=COLUMN_SCALAR, type=ShortText)
+        namecol = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                         type=DataType.ShortText)
+        passwordcol = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                             type=DataType.ShortText)
 
         class Tb1(Table):
             name = namecol
             password = passwordcol
             address = 'address'
 
-        self.assertIs(Tb1.__tableflags__, TABLE_HASH_KEY)
-        self.assertIs(Tb1.__key_type__, ShortText)
+        self.assertIs(Tb1.__tableflags__, TableFlags.TABLE_HASH_KEY)
+        self.assertIs(Tb1.__key_type__, DataType.ShortText)
+        self.assertIs(Tb1.__default_tokenizer__, None)
         self.assertEqual(Tb1.__tablename__, 'Tb1')
         self.assertEqual(str(Tb1), 'table_create --name Tb1 --flags ' \
                 'TABLE_HASH_KEY --key_type ShortText')
         self.assertListEqual(Tb1.columns, [namecol, passwordcol])
         self.assertListEqual(Table._tables, [Tb1])
 
-        sitecol = Column(flags=COLUMN_SCALAR, type=ShortText)
-        addresscol = Column(flags=COLUMN_SCALAR, type=ShortText)
+        sitecol = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                         type=DataType.ShortText)
+        addresscol = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                            type=DataType.ShortText)
 
         class Tb2(Table):
-            __tableflags__ = TABLE_PAT_KEY
-            __key_type__ = UInt32
+            __tableflags__ = TableFlags.TABLE_PAT_KEY
+            __key_type__ = DataType.UInt32
+            __default_tokenizer__ = Tokenizer.TokenBigram
             site = sitecol
             address = addresscol
 
-        self.assertIs(Tb2.__tableflags__, TABLE_PAT_KEY)
-        self.assertIs(Tb2.__key_type__, UInt32)
+        self.assertIs(Tb2.__tableflags__, TableFlags.TABLE_PAT_KEY)
+        self.assertIs(Tb2.__key_type__, DataType.UInt32)
         self.assertEqual(Tb2.__tablename__, 'Tb2')
         self.assertEqual(str(Tb2), 'table_create --name Tb2 --flags ' \
-                'TABLE_PAT_KEY --key_type UInt32')
+                'TABLE_PAT_KEY --key_type UInt32 --default_tokenizer ' \
+                'TokenBigram')
         self.assertListEqual(Tb2.columns, [sitecol, addresscol])
         self.assertListEqual(Table._tables, [Tb1, Tb2])
 
+        class Tb3(Table):
+            __tableflags__ = TableFlags.TABLE_NO_KEY
+            site = sitecol
+            address = addresscol
+
+        self.assertIs(Tb3.__tableflags__, TableFlags.TABLE_NO_KEY)
+        self.assertEqual(Tb3.__tablename__, 'Tb3')
+        self.assertEqual(str(Tb3), 'table_create --name Tb3 --flags ' \
+                         'TABLE_NO_KEY')
+        self.assertListEqual(Tb3.columns, [sitecol, addresscol])
+        self.assertListEqual(Table._tables, [Tb1, Tb2, Tb3])
+
     def test_bind(self):
         Table = tablebase()
         self.assertRaises(TypeError, Table.bind, 'dummy')
@@ -223,10 +288,12 @@ class TestTable(GroongaTestBase):
         Table = tablebase()
 
         class Tb1(Table):
-            name = Column(flags=COLUMN_SCALAR, type=ShortText)
+            name = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                          type=DataType.ShortText)
 
         class Tb2(Table):
-            word = Column(flags=COLUMN_SCALAR, type=ShortText)
+            word = Column(flags=ColumnFlags.COLUMN_SCALAR,
+                          type=DataType.ShortText)
 
         grn = Groonga()
         Table.bind(grn)
@@ -242,7 +309,7 @@ class TestTable(GroongaTestBase):
                      'TABLE_HASH_KEY|PERSISTENT',
                      'ShortText',
                      'null'],
-                    [258, 'Tb2', GroongaTestBase.DB_PATH + '.0000102',
+                    [257, 'Tb2', GroongaTestBase.DB_PATH + '.0000101',
                      'TABLE_HASH_KEY|PERSISTENT',
                      'ShortText',
                      'null']]
@@ -258,9 +325,9 @@ class TestTable(GroongaTestBase):
                      ['range', 'ShortText'],
                      ['source', 'ShortText']],
                     [256, '_key', '', '', 'COLUMN_SCALAR', 'Tb1', 'ShortText', []],
-                    [257,
+                    [258,
                      'name',
-                     GroongaTestBase.DB_PATH + '.0000101',
+                     GroongaTestBase.DB_PATH + '.0000102',
                      'var',
                      'COLUMN_SCALAR|PERSISTENT',
                      'Tb1',
@@ -277,7 +344,7 @@ class TestTable(GroongaTestBase):
                      ['domain', 'ShortText'],
                      ['range', 'ShortText'],
                      ['source', 'ShortText']],
-                    [258, '_key', '', '', 'COLUMN_SCALAR', 'Tb2', 'ShortText', []],
+                    [257, '_key', '', '', 'COLUMN_SCALAR', 'Tb2', 'ShortText', []],
                     [259,
                      'word',
                      GroongaTestBase.DB_PATH + '.0000103',
@@ -807,6 +874,354 @@ class TestTable(GroongaTestBase):
         self.assertListEqual(stored[1], expected)
 
 
+class TestSuggestTable(GroongaTestBase):
+    def setUp(self):
+        super(TestSuggestTable, self).setUp()
+        super(TestSuggestTable, self).tearDownClass()
+        super(TestSuggestTable, self).setUpClass()
+
+    def tearDown(self):
+        super(TestSuggestTable, self).tearDown()
+        super(TestSuggestTable, self).tearDownClass()
+
+    def _sendquery(self, cmd):
+        proc = Popen('groonga -c', shell=True, stdin=PIPE, stdout=PIPE,
+                stderr=PIPE)
+        result = proc.communicate(cmd.encode('utf-8'))[0]
+        proc.wait()
+        return result.decode('utf-8')
+
+    def _load(self, count):
+        fixture = self.loadfixture('_suggest')
+        data = json.dumps(fixture)
+        for i in range(count):
+            self._sendquery("load --table event_query --input_type json " \
+                "--each 'suggest_preparer(_id, type, item, sequence, time, " \
+                "pair_query)'\n%s" % (data))
+
+    def test_create_all(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        result = json.loads(self._sendquery('table_list'))
+        expected = [[['id', 'UInt32'],
+                     ['name', 'ShortText'],
+                     ['path', 'ShortText'],
+                     ['flags', 'ShortText'],
+                     ['domain', 'ShortText'],
+                     ['range', 'ShortText']],
+                    [259, 'bigram', GroongaTestBase.DB_PATH + '.0000103',
+                     'TABLE_PAT_KEY|KEY_NORMALIZE|PERSISTENT',
+                     'ShortText',
+                     'null'],
+                    [264, 'event_query', GroongaTestBase.DB_PATH + '.0000108',
+                     'TABLE_NO_KEY|PERSISTENT',
+                     'null',
+                     'null'],
+                    [258, 'event_type', GroongaTestBase.DB_PATH + '.0000102',
+                     'TABLE_HASH_KEY|PERSISTENT',
+                     'ShortText',
+                     'null'],
+                    [261, 'item_query', GroongaTestBase.DB_PATH + '.0000105',
+                     'TABLE_PAT_KEY|KEY_NORMALIZE|PERSISTENT',
+                     'ShortText',
+                     'null'],
+                    [262, 'kana', GroongaTestBase.DB_PATH + '.0000106',
+                     'TABLE_PAT_KEY|KEY_NORMALIZE|PERSISTENT',
+                     'ShortText',
+                     'null'],
+                    [260, 'pair_query', GroongaTestBase.DB_PATH + '.0000104',
+                     'TABLE_HASH_KEY|PERSISTENT',
+                     'UInt64',
+                     'null'],
+                    [263, 'sequence_query', GroongaTestBase.DB_PATH + '.0000107',
+                     'TABLE_HASH_KEY|PERSISTENT',
+                     'ShortText',
+                     'null'],
+                    ]
+        self.assertListEqual(result[1], expected)
+
+    def test_load(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        data = [event_query(time=12, sequence='deadbeef', item='s'),
+                event_query(time=13, sequence='deadbeef', item='se'),
+                event_query(time=14, sequence='deadbeef', item='sea',
+                            type='submit')]
+        result = event_query.load(data)
+        self.assertEqual(result, 3)
+        stored = json.loads(self._sendquery('select --table event_query'))
+        expected = [[[3],
+                     [['_id', 'UInt32'],
+                      ['item', 'item_query'],
+                      ['sequence', 'sequence_query'],
+                      ['time', 'Time'],
+                      ['type', 'event_type']],
+                     [1, 's', 'deadbeef', 12.0, ''],
+                     [2, 'se', 'deadbeef', 13.0, ''],
+                     [3, 'sea', 'deadbeef', 14.0, 'submit']]]
+        self.assertListEqual(stored[1], expected)
+
+        data = [event_query(time=22, sequence='foobar', item='mou'),
+                event_query(time=23, sequence='foobar', item='moun',
+                            type='submit'),
+                event_query(time=24, sequence='foobar', item='mountain',
+                            type='submit')]
+        result = event_query.load(data)
+        self.assertEqual(result, 3)
+        stored = json.loads(self._sendquery('select --table event_query'))
+        expected = [[[6],
+                     [['_id', 'UInt32'],
+                      ['item', 'item_query'],
+                      ['sequence', 'sequence_query'],
+                      ['time', 'Time'],
+                      ['type', 'event_type']],
+                     [1, 's', 'deadbeef', 12.0, ''],
+                     [2, 'se', 'deadbeef', 13.0, ''],
+                     [3, 'sea', 'deadbeef', 14.0, 'submit'],
+                     [4, 'mou', 'foobar', 22.0, ''],
+                     [5, 'moun', 'foobar', 23.0, 'submit'],
+                     [6, 'mountain', 'foobar', 24.0, 'submit']]]
+        self.assertListEqual(stored[1], expected)
+
+        stored = json.loads(self._sendquery('select --table item_query'))
+        expected = [[[6],
+                     [['_id', 'UInt32'],
+                      ['_key', 'ShortText'],
+                      ['boost', 'Int32'],
+                      ['buzz', 'Int32'],
+                      ['co', 'pair_query'],
+                      ['freq', 'Int32'],
+                      ['freq2', 'Int32'],
+                      ['kana', 'kana'],
+                      ['last', 'Time']],
+                     [4, 'mou', 0, 0, 1, 1, 0, [], 22.0],
+                     [5, 'moun', 0, 0, 1, 1, 1, [], 23.0],
+                     [6, 'mountain', 0, 0, 0, 1, 1, [], 24.0],
+                     [1, 's', 0, 0, 1, 1, 0, [], 12.0],
+                     [2, 'se', 0, 0, 1, 1, 0, [], 13.0],
+                     [3, 'sea', 0, 0, 0, 1, 1, [], 14.0]]]
+        self.assertListEqual(stored[1], expected)
+
+    def test_suggest_all(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        # for default frequency threshold by groonga
+        self._load(count=100)
+
+        result = item_query.suggest('en').all()
+        self.assertIsInstance(result, GroongaSuggestResults)
+        expected_complete = [
+            ('engine', 100),
+            ]
+        self.assertEqual(len(result.complete), len(expected_complete))
+        for i, r in enumerate(result.complete):
+            self.assertEqual(r._key, expected_complete[i][0])
+            self.assertEqual(r._score, expected_complete[i][1])
+        self.assertListEqual(result.correct, [])
+        self.assertListEqual(result.suggest, [])
+
+        result = item_query.suggest('sea').all()
+        self.assertIsInstance(result, GroongaSuggestResults)
+        expected_complete = [
+            ('search', 101),
+            ('search engine', 101),
+            ]
+        self.assertEqual(len(result.complete), len(expected_complete))
+        for i, r in enumerate(result.complete):
+            self.assertEqual(r._key, expected_complete[i][0])
+            self.assertEqual(r._score, expected_complete[i][1])
+
+    def test_suggest_get(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        # for default frequency threshold by groonga
+        self._load(count=100)
+
+        result = item_query.suggest('en').get('complete')
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = [
+            ('engine', 100),
+            ]
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+        result = item_query.suggest('en').get('correct')
+        self.assertListEqual(result, [])
+
+        result = item_query.suggest('en').get('suggest')
+        self.assertListEqual(result, [])
+
+        with self.assertRaises(KeyError):
+            item_query.suggest('en').get('dummy')
+
+    def test_suggest___getitem__(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        # for default frequency threshold by groonga
+        self._load(count=100)
+
+        result = item_query.suggest('en')['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = [
+            ('engine', 100),
+            ]
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+        result = item_query.suggest('en')['correct']
+        self.assertListEqual(result, [])
+
+        result = item_query.suggest('en')['suggest']
+        self.assertListEqual(result, [])
+
+        with self.assertRaises(KeyError):
+            item_query.suggest('en')['dummy']
+
+    def test_suggest_with_types(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+        # for default frequency threshold by groonga
+        self._load(count=100)
+
+        values = [
+            (SuggestType.complete, 'en'),
+            (SuggestType.correct, 'saerch'),
+            (SuggestType.suggest, 'search'),
+            (SuggestType.complete | SuggestType.correct, 'search'),
+            (SuggestType.complete | SuggestType.suggest, 'search'),
+            (SuggestType.complete | SuggestType.correct | SuggestType.suggest,
+                'search')]
+        expects = [
+            # (complete, correct, suggest)
+            (['engine'], [], []),
+            ([], ['search'], []),
+            ([], [], ['search engine', 'web search realtime']),
+            (['search', 'search engine'], [], []),
+            (['search', 'search engine'], [],
+             ['search engine', 'web search realtime']),
+            (['search', 'search engine'], [],
+             ['search engine', 'web search realtime'])
+            ]
+        for i, (types, query) in enumerate(values):
+            result = item_query.suggest(query).types(types).all()
+            self.assertIsInstance(result, GroongaSuggestResults)
+            expected = expects[i]
+
+            self.assertEqual(len(result.complete), len(expected[0]),
+                    'complete, loop %d' % i)
+            for j, r in enumerate(result.complete):
+                self.assertEqual(r._key, expected[0][j])
+            self.assertEqual(len(result.correct), len(expected[1]),
+                    'correct, loop %d' % i)
+            for j, r in enumerate(result.correct):
+                self.assertEqual(r._key, expected[1][j])
+            self.assertEqual(len(result.suggest), len(expected[2]),
+                    'suggest, loop %d' % i)
+            for j, r in enumerate(result.suggest):
+                self.assertEqual(r._key, expected[2][j])
+
+    def test_suggest_with_frequency_threshold(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+
+        self._load(count=3)
+        result = item_query.suggest('en').frequency_threshold(2)['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = [
+            ('engine', 3),
+            ]
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+        result = item_query.suggest('en').frequency_threshold(3)['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = [
+            ('engine', 3),
+            ]
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+        result = item_query.suggest('en').frequency_threshold(3)['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = [
+            ('engine', 3),
+            ]
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+        result = item_query.suggest('en').frequency_threshold(4)['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = []
+        try:
+            self.assertEqual(len(result), len(expected))
+            for i, r in enumerate(result):
+                self.assertEqual(r._key, expected[i][0])
+                self.assertEqual(r._score, expected[i][1])
+        except AssertionError:
+            # Expected failure. bugs in groonga?
+            pass
+
+        result = item_query.suggest('en').frequency_threshold(5)['complete']
+        self.assertIsInstance(result, GroongaSuggestResult)
+        expected = []
+        self.assertEqual(len(result), len(expected))
+        for i, r in enumerate(result):
+            self.assertEqual(r._key, expected[i][0])
+            self.assertEqual(r._score, expected[i][1])
+
+    def test_suggest_with_conditional_probability_threshold(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+
+        query = item_query.suggest('en'). \
+                conditional_probability_threshold(1)
+        self.assertEqual(str(query),
+                'suggest --table "item_query" --column ' \
+                '"kana" --types "complete" ' \
+                '--conditional_probability_threshold 1.0 --query "en"')
+
+        query = item_query.suggest('en'). \
+                conditional_probability_threshold(0.2)
+        self.assertEqual(str(query),
+                'suggest --table "item_query" --column ' \
+                '"kana" --types "complete" ' \
+                '--conditional_probability_threshold 0.2 --query "en"')
+
+
+    def test_suggest_with_prefix_search(self):
+        grn = Groonga()
+        SuggestTable.bind(grn)
+        SuggestTable.create_all()
+
+        query = item_query.suggest('en').prefix_search(True)
+        self.assertEqual(str(query),
+                'suggest --table "item_query" --column ' \
+                '"kana" --types "complete" --prefix_search yes --query "en"')
+
+        query = item_query.suggest('en').prefix_search(False)
+        self.assertEqual(str(query),
+                'suggest --table "item_query" --column ' \
+                '"kana" --types "complete" --prefix_search no --query "en"')
+
+
 def main():
     unittest.main()
 
